package ch.hslu.edu.enapp.webshop.boundary;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateless;
import javax.faces.context.FacesContext;
import javax.inject.Inject;

import ch.hslu.edu.enapp.webshop.common.CustomerServiceLocal;
import ch.hslu.edu.enapp.webshop.common.CustomerServiceRemote;
import ch.hslu.edu.enapp.webshop.common.dto.AuthgroupDTO;
import ch.hslu.edu.enapp.webshop.common.dto.CustomerDTO;
import ch.hslu.edu.enapp.webshop.control.AuthgroupManager;
import ch.hslu.edu.enapp.webshop.control.CustomerManager;
import ch.hslu.edu.enapp.webshop.control.UserGroupManager;
import ch.hslu.edu.enapp.webshop.entity.Authgroup;
import ch.hslu.edu.enapp.webshop.entity.Customer;
import ch.hslu.edu.enapp.webshop.entity.UserGroup;
import ch.hslu.edu.enapp.webshop.entity.UserGroupPK;

/**
 * Session Bean implementation class CustomerService
 */
@Stateless
@Local(CustomerServiceLocal.class)
@Remote(CustomerServiceRemote.class)
public class CustomerService implements CustomerServiceRemote, CustomerServiceLocal, Serializable {

	private static final long serialVersionUID = 1L;

	@Inject
	CustomerManager customerManager;

	@Inject
	UserGroupManager userGroupManager;

	@Inject
	AuthgroupManager authgroupManager;

	/**
	 * Default constructor.
	 */
	public CustomerService() {
		// empty
	}

	@Override
	public CustomerDTO getCustomerByUsername(final String username) {
		final List<Customer> customer = customerManager.getCustomerByUsername(username);
		if (customer.size() < 1) {
			return new CustomerDTO();
		}
		return CustomerConverter.createDTOFromEntity(customer.get(0));
	}

	@Override
	public String getUsername() {
		return FacesContext.getCurrentInstance().getExternalContext().getUserPrincipal().getName();
	}

	@Override
	public CustomerDTO getCustomer() {
		final String username = getUsername();
		if (username == null) {
			return new CustomerDTO();
		}
		if (username.equals("")) {
			return new CustomerDTO();
		}

		return getCustomerByUsername(username);
	}

	@Override
	public boolean updateCustomer(final CustomerDTO customerDTO) {
		final Customer customer = CustomerConverter.createEntityFromDTO(customerDTO);
		try {
			customerManager.updateCustomer(customer);
		} catch (final Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}
		return true;
	}

	@Override
	public boolean createCustomer(final CustomerDTO customerDTO) {
		final Customer customer = CustomerConverter.createEntityFromDTO(customerDTO);
		try {
			customerManager.createCustomer(customer);
		} catch (final Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}

		return true;
	}

	@Override
	public boolean createCustomerWithLogin(final CustomerDTO customerDTO) {
		final Customer customer = CustomerConverter.createEntityFromDTO(customerDTO);
		try {
			customerManager.createCustomer(customer);

			final UserGroupPK pk = new UserGroupPK();
			pk.setIdgroup("1");
			pk.setIduser(customer.getId());
			final UserGroup permission = new UserGroup();
			permission.setId(pk);
			userGroupManager.createUserGroup(permission);

		} catch (final Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return false;
		}

		return true;
	}

	@Override
	public List<CustomerDTO> getCustomers() {
		final List<CustomerDTO> customerList = new ArrayList<CustomerDTO>();
		for (final Customer customer : customerManager.getCustomer()) {
			customerList.add(CustomerConverter.createDTOFromEntity(customer));
		}

		return customerList;
	}

	@Override
	public List<AuthgroupDTO> getPermissionGroups(final CustomerDTO customer) {
		final List<AuthgroupDTO> groups = new ArrayList<AuthgroupDTO>();
		for (final UserGroup userGroup : userGroupManager.getUserGroupById_iduser(customer.getId())) {
			final List<Authgroup> authgroup = authgroupManager.getAuthgroupByIdauthgroup(userGroup.getId().getIdgroupInt());
			if (!authgroup.isEmpty()) {
				groups.add(AuthGroupConverter.createDTOFromEntity(authgroup.get(0)));
			}
		}

		return groups;
	}

	@Override
	public CustomerDTO getCustomerById(final int customerId) {
		final List<Customer> customer = customerManager.getCustomerById(customerId);
		if (customer.size() < 1) {
			return new CustomerDTO();
		}
		return CustomerConverter.createDTOFromEntity(customer.get(0));
	}

	@Override
	public boolean isMemberOfGroup(final CustomerDTO customer, final String groupname) {
		// System.out.println("Check Customer: " + customer.getUsername() + " in Group: " + groupname);

		final List<Authgroup> authGroups = authgroupManager.getAuthgroupByGroupname(groupname);
		if (authGroups.isEmpty()) {
			return false;
		}

		final List<AuthgroupDTO> memberGroups = getPermissionGroups(customer);
		if (memberGroups.isEmpty()) {
			return false;
		}

		for (final AuthgroupDTO group : memberGroups) {
			if (group.getIdauthgroup() == authGroups.get(0).getIdauthgroup()) {
				return true;
			}
		}

		return false;
	}

	@Override
	public boolean isAdmin(final CustomerDTO customer) {
		final boolean result = isMemberOfGroup(customer, "adminRole");
		// System.out.println("Customer: " + customer.getUsername() + " isAdmin: " + result);
		return result;
	}

}
