package ch.hslu.edu.enapp.webshop.boundary;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.ejb.Stateful;

import ch.hslu.edu.enapp.webshop.common.ShoppingCardServiceLocal;
import ch.hslu.edu.enapp.webshop.common.ShoppingCardServiceRemote;
import ch.hslu.edu.enapp.webshop.common.dto.ProductDTO;

/**
 * Session Bean implementation class ShoppingCardService
 */
@Stateful
@Local(ShoppingCardServiceLocal.class)
@Remote(ShoppingCardServiceRemote.class)
public class ShoppingCardService implements ShoppingCardServiceRemote, ShoppingCardServiceLocal, Serializable {

	private static final long serialVersionUID = 1L;
	private final Map<ProductDTO, Integer> shoppingCart;
	private double totalPrice;

	/**
	 * Default constructor.
	 */
	public ShoppingCardService() {
		shoppingCart = new HashMap<ProductDTO, Integer>();
	}

	@Override
	public void addItem(final ProductDTO product) {
		if (shoppingCart.containsKey(product)) {
			if (getAmountOfProduct(product) < 99) {
				shoppingCart.put(product, shoppingCart.get(product) + 1);
			}
		} else {
			shoppingCart.put(product, 1);
		}
	}

	@Override
	public void removeItem(final ProductDTO product) {
		if (shoppingCart.get(product) > 1) {
			shoppingCart.put(product, shoppingCart.get(product) - 1);
		} else {
			shoppingCart.remove(product);
		}
	}

	@Override
	public List<ProductDTO> getItems() {
		final List<ProductDTO> products = new ArrayList<ProductDTO>();
		products.addAll(shoppingCart.keySet());
		return products;
	}

	@Override
	public int getAmountOfProduct(final ProductDTO productToRetrieve) {
		return shoppingCart.get(productToRetrieve);
	}

	@Override
	public double calcTotal() {
		totalPrice = 0.0;
		for (final ProductDTO product : getItems()) {
			totalPrice += product.getUnitprice().doubleValue() * getAmountOfProduct(product);
			// System.out.println("Sum: " + sum + ", Total: " + totalPrice);
		}
		return totalPrice;
	}

	@Override
	public Map<ProductDTO, Integer> getShoppingCard() {
		return shoppingCart;
	}

	@Override
	public void clear() {
		shoppingCart.clear();
	}
}
