package ch.hslu.edu.enapp.webshop.control;

import com.ibm.jpa.web.JPAManager;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import com.ibm.jpa.web.NamedQueryTarget;
import com.ibm.jpa.web.Action;
import javax.persistence.PersistenceUnit;
import ch.hslu.edu.enapp.webshop.entity.Purchase;
import java.util.List;
import javax.persistence.Query;
import javax.persistence.TemporalType;
import java.util.Date;

@SuppressWarnings("unchecked")
@JPAManager(targetEntity = ch.hslu.edu.enapp.webshop.entity.Purchase.class)
public class PurchaseManager {

	@PersistenceUnit
	private EntityManagerFactory emf;

	public PurchaseManager() {
	
	}

	private EntityManager getEntityManager() {
		return emf.createEntityManager();
	}

	@Action(Action.ACTION_TYPE.CREATE)
	public void createPurchase(Purchase purchase) throws Exception {
		EntityManager em = getEntityManager();
		try {
			em.persist(purchase);
		} finally {
			em.close();
		}
	}

	@Action(Action.ACTION_TYPE.DELETE)
	public void deletePurchase(Purchase purchase) throws Exception {
		EntityManager em = getEntityManager();
		try {
			purchase = em.merge(purchase);
			em.remove(purchase);
		} finally {
			em.close();
		}
	}

	@Action(Action.ACTION_TYPE.UPDATE)
	public void updatePurchase(Purchase purchase) throws Exception {
		EntityManager em = getEntityManager();
		try {
			purchase = em.merge(purchase);
		} finally {
			em.close();
		}
	}

	@Action(Action.ACTION_TYPE.FIND)
	public Purchase findPurchaseById(int id) {
		Purchase purchase = null;
		EntityManager em = getEntityManager();
		try {
			purchase = (Purchase) em.find(Purchase.class, id);
		} finally {
			em.close();
		}
		return purchase;
	}

	@Action(Action.ACTION_TYPE.NEW)
	public Purchase getNewPurchase() {
	
		Purchase purchase = new Purchase();
	
		return purchase;
	}

	@NamedQueryTarget("getPurchaseByNAVOrderId")
	public List<Purchase> getPurchaseByNAVOrderId(String NAVOrderId) {
		EntityManager em = getEntityManager();
		List<Purchase> results = null;
		try {
			Query query = em.createNamedQuery("getPurchaseByNAVOrderId");
			query.setParameter("NAVOrderId", NAVOrderId);
			results = (List<Purchase>) query.getResultList();
		} finally {
			em.close();
		}
		return results;
	}

	@NamedQueryTarget("getPurchaseByCorrelationId")
	public List<Purchase> getPurchaseByCorrelationId(String correlationId) {
		EntityManager em = getEntityManager();
		List<Purchase> results = null;
		try {
			Query query = em.createNamedQuery("getPurchaseByCorrelationId");
			query.setParameter("correlationId", correlationId);
			results = (List<Purchase>) query.getResultList();
		} finally {
			em.close();
		}
		return results;
	}

	@NamedQueryTarget("getPurchaseByStatus")
	public List<Purchase> getPurchaseByStatus(String status) {
		EntityManager em = getEntityManager();
		List<Purchase> results = null;
		try {
			Query query = em.createNamedQuery("getPurchaseByStatus");
			query.setParameter("status", status);
			results = (List<Purchase>) query.getResultList();
		} finally {
			em.close();
		}
		return results;
	}

	@NamedQueryTarget("getPurchaseByPayid")
	public List<Purchase> getPurchaseByPayid(int payid) {
		EntityManager em = getEntityManager();
		List<Purchase> results = null;
		try {
			Query query = em.createNamedQuery("getPurchaseByPayid");
			query.setParameter("payid", payid);
			results = (List<Purchase>) query.getResultList();
		} finally {
			em.close();
		}
		return results;
	}

	@NamedQueryTarget("getPurchaseByNcstatus")
	public List<Purchase> getPurchaseByNcstatus(int ncstatus) {
		EntityManager em = getEntityManager();
		List<Purchase> results = null;
		try {
			Query query = em.createNamedQuery("getPurchaseByNcstatus");
			query.setParameter("ncstatus", ncstatus);
			results = (List<Purchase>) query.getResultList();
		} finally {
			em.close();
		}
		return results;
	}

	@NamedQueryTarget("getPurchaseByNcerrorplus")
	public List<Purchase> getPurchaseByNcerrorplus(String ncerrorplus) {
		EntityManager em = getEntityManager();
		List<Purchase> results = null;
		try {
			Query query = em.createNamedQuery("getPurchaseByNcerrorplus");
			query.setParameter("ncerrorplus", ncerrorplus);
			results = (List<Purchase>) query.getResultList();
		} finally {
			em.close();
		}
		return results;
	}

	@NamedQueryTarget("getPurchaseByNcerror")
	public List<Purchase> getPurchaseByNcerror(int ncerror) {
		EntityManager em = getEntityManager();
		List<Purchase> results = null;
		try {
			Query query = em.createNamedQuery("getPurchaseByNcerror");
			query.setParameter("ncerror", ncerror);
			results = (List<Purchase>) query.getResultList();
		} finally {
			em.close();
		}
		return results;
	}

	@NamedQueryTarget("getPurchaseByDatetime")
	public List<Purchase> getPurchaseByDatetime(Date datetime) {
		EntityManager em = getEntityManager();
		List<Purchase> results = null;
		try {
			Query query = em.createNamedQuery("getPurchaseByDatetime");
			query.setParameter("datetime", datetime, TemporalType.TIMESTAMP);
			results = (List<Purchase>) query.getResultList();
		} finally {
			em.close();
		}
		return results;
	}

	@NamedQueryTarget("getPurchaseByCustomerid")
	public List<Purchase> getPurchaseByCustomerid(int customerid) {
		EntityManager em = getEntityManager();
		List<Purchase> results = null;
		try {
			Query query = em.createNamedQuery("getPurchaseByCustomerid");
			query.setParameter("customerid", customerid);
			results = (List<Purchase>) query.getResultList();
		} finally {
			em.close();
		}
		return results;
	}

	@NamedQueryTarget("getPurchaseById")
	public List<Purchase> getPurchaseById(int id) {
		EntityManager em = getEntityManager();
		List<Purchase> results = null;
		try {
			Query query = em.createNamedQuery("getPurchaseById");
			query.setParameter("id", id);
			results = (List<Purchase>) query.getResultList();
		} finally {
			em.close();
		}
		return results;
	}

	@NamedQueryTarget("getPurchase")
	public List<Purchase> getPurchase() {
		EntityManager em = getEntityManager();
		List<Purchase> results = null;
		try {
			Query query = em.createNamedQuery("getPurchase");
			results = (List<Purchase>) query.getResultList();
		} finally {
			em.close();
		}
		return results;
	}

}